//+------------------------------------------------------------------+
//|                                            Twiggs Money Flow.mq4 |
//|                                                                  |
//| smoothing done by mladen                                         |
//+------------------------------------------------------------------+
#property copyright " Mr. Colin Twiggs & mladen"
#property link      "http://www.incrediblecharts.com/technical/twiggs_money_flow.php"


#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  Lime

//
//
//
//
//

extern string TimeFrame = "Current time frame";
extern int    Prd    = 21;
extern double Smooth = 8;

//
//
//
//
//

double ExtMapBuffer1[];
double WildersBuffer[];
double WVBuffer[];
double ADVBuffer[];
double SmV[];
double SmA[];
string indicatorFileName;
bool   returnBars;
int    timeFrame;



//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{

   IndicatorBuffers(6);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexBuffer(1,WildersBuffer);
   SetIndexBuffer(2,WVBuffer);
   SetIndexBuffer(3,ADVBuffer);
   SetIndexBuffer(4,SmV);
   SetIndexBuffer(5,SmA);
   SetLevelValue(0,0.0);
   SetLevelValue(1,0.1);
   SetLevelValue(2,0.25);
   SetLevelValue(3,-0.1);
   SetLevelValue(4,-0.25);
      timeFrame         = stringToTimeFrame(TimeFrame);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars"; if (returnBars) return(0);
      IndicatorShortName(timeFrameToString(timeFrame)+" Twiggs Money Flow");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int deinit()
{

   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { ExtMapBuffer1[0] = limit+1; return(0); }
         if (timeFrame!=Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            for (int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  ExtMapBuffer1[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Prd,Smooth,0,y);
            }
            return(0);                  
         }

     //
     //
     //
     //
     //

     for (i = limit; i >= 0; i--)
     {

       double TRH = iHigh(NULL, 0, iHighest(NULL, 0, MODE_HIGH, 1, i)) + 1*Point;
       double TRL = iLow(NULL, 0, iLowest(NULL, 0, MODE_LOW, 1, i)) - 1*Point;
       double TR  =  TRH-TRL;
       if (TR==0) { ADVBuffer[i] = ((Close[i]-TRL)-(TRH-Close[i]))/ (999999)*Volume[i];  }
       else       { ADVBuffer[i] = ((Close[i]-TRL)-(TRH-Close[i]))/ (TR)*Volume[i];      }
      
        WVBuffer[i]     = Volume[i] + (iVolume(NULL,0,i)*0);  
        WildersBuffer[i]= iMA(NULL,0,(Prd*2)-1,0,MODE_EMA,PRICE_CLOSE,i);
   
      
    }

                  
      for (i = limit; i >= 0; i--)
      {
        SmV[i] = iMAOnArray(WVBuffer,0,(Prd*2)-1,0,MODE_EMA,i);
        SmA[i]= iMAOnArray(ADVBuffer,0,(Prd*2)-1,0,MODE_EMA,i);
      
       if (TRH != 0)
            ExtMapBuffer1[i] = iSsm(SmA[i]/SmV[i],Smooth,i);
       else ExtMapBuffer1[i] = iSsm(0            ,Smooth,i);
   }
return(0);
}
//+------------------------------------------------------------------+



//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workSsm[][2];
#define _price  0
#define _ssm    1

double workSsmCoeffs[][4];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define Pi 3.14159265358979323846264338327950288

//
//
//
//
//

double iSsm(double price, double period, int i, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workSsm,0) !=Bars)                 ArrayResize(workSsm,Bars);
   if (ArrayRange(workSsmCoeffs,0) < (instanceNo+1)) ArrayResize(workSsmCoeffs,instanceNo+1);
   if (workSsmCoeffs[instanceNo][_period] != period)
   {
      workSsmCoeffs[instanceNo][_period] = period;
      double a1 = MathExp(-1.414*Pi/period);
      double b1 = 2.0*a1*MathCos(1.414*Pi/period);
         workSsmCoeffs[instanceNo][_c2] = b1;
         workSsmCoeffs[instanceNo][_c3] = -a1*a1;
         workSsmCoeffs[instanceNo][_c1] = 1.0 - workSsmCoeffs[instanceNo][_c2] - workSsmCoeffs[instanceNo][_c3];
   }

   //
   //
   //
   //
   //

      i = Bars-i-1;
      int s = instanceNo*2;   
          workSsm[i][s+_price] = price;
          workSsm[i][s+_ssm]   = workSsmCoeffs[instanceNo][_c1]*(workSsm[i][s+_price]+workSsm[i-1][s+_price])/2.0 + 
                                 workSsmCoeffs[instanceNo][_c2]*workSsm[i-1][s+_ssm]                              + 
                                 workSsmCoeffs[instanceNo][_c3]*workSsm[i-2][s+_ssm];
   return(workSsm[i][s+_ssm]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}


//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}